##################################
### SearchInfo.tcl             ###
### Version 1.0                ###
### By Wcc                     ###
### wcc@techmonkeys.org        ###
### http://www.dawgtcl.com:81/ ###
### EFnet #|DAWG|Tcl           ###
##################################

#####################################################################
## This script provides tools for displaying information about the ##
## bot's users (ICQ numbers, email addresses, phone numbers, etc). ##
## The userinfo.tcl script (provided in the scripts directory of   ##
## your eggdrop) must be loaded to use this script.                ##
#####################################################################

##############
## COMMANDS ##
##################################################################################
## DCC ## .list<type> [-email <address>|-dcc <nick>] [-csv] [-flag <flag>]      ##
######### Shows you a list of users' whois feilds of <type>. You can add types  ##
######### by editing the settings in userinfo.tcl. The -email option sends the  ##
######### data via email. The -dcc option sends the data in a file via dcc. If  ##
######### neither option is specified, the information will be displayed in dcc ##
######### chat. If you specify the -csv (comma separated values) option, the    ##
######### data will be displayed (or sent) all on a single line separated by    ##
######### commas. For example: "me@whatev.er, blah@blah.cc". If you specify the ##
######### -flag option, it will only match users with <flag>.                   ##
######### --------------------------------------------------------------------- ##
######### .show<type> <user>                                                    ##
######### Shows a user's <type> data.                                           ##
######### --------------------------------------------------------------------- ##
######### .search<type> <search-string>                                         ##
######### Search all users' whois fields for <search-string>.                   ##
######### For example: .searchemail *wcc* would show any email addresses that   ##
######### contain 'wcc'.                                                        ##
##################################################################################

##########################################################
## Just load the script, edit the settings, and rehash. ##
##########################################################

############################################################
# Set the flag required to use the script's commands here. #
############################################################

set searchinfo_setting(flag) "+m"

##############################################################################
# Any user with this flag will be ignored from all searches ("" to disable). #
##############################################################################

set searchinfo_setting(ignore) "+R"

###################################
# Enable use of bold in dcc chat? #
###################################

set searchinfo_setting(bold) 1

##############################################
# Prefix "SEARCHINFO:" in dcc chat messages? #
##############################################

set searchinfo_setting(SEARCHINFO:) 1

####################
# Code begins here #
####################

if {![string match 1.6.* $version]} { putlog "\002SEARCHINFO:\002 \002WARNING:\002 This script is intended to run on eggdrop 1.6.x or later." }
if {[info tclversion] < 8.2} { putlog "\002SEARCHINFO:\002 \002WARNING:\002 This script is intended to run on Tcl Version 8.2 or later." }
if {![info exists userinfo_loaded] || !$userinfo_loaded} { putlog "\002SEARCHINFO:\002 \002WARNING:\002 This script requires userinfo.tcl You can find this in your eggdrop scripts directory." }

foreach bt [string tolower ${userinfo-fields}] { bind dcc $searchinfo_setting(flag) "list$bt" [list searchinfo_list $bt] }
foreach bt [string tolower ${userinfo-fields}] { bind dcc $searchinfo_setting(flag) "show$bt" [list searchinfo_show $bt] }
foreach bt [string tolower ${userinfo-fields}] { bind dcc $searchinfo_setting(flag) "search$bt" [list searchinfo_search $bt] }

proc searchinfo_dopre {} {
	if {!$::searchinfo_setting(SEARCHINFO:)} { return "" }
	if {!$::searchinfo_setting(bold)} { return "SEARCHINFO: " }
	return "\002SEARCHINFO:\002 "
}

proc searchinfo_list {type hand idx text} {
	set list ""
	if {[set index [lsearch -exact [set text [split [string tolower $text]]] "-email"]] != -1} {
		if {[llength $text] <= 1} { putdcc $idx "[searchinfo_dopre]You must specify an email address." ; return }
		set email [join [lrange $text [expr $index + 1] [expr $index + 1]]]
	}
	if {[set index [lsearch -exact $text "-dcc"]] != -1} {
		if {[llength $text] <= 1} { putdcc $idx "[searchinfo_dopre]You must specify a dcc nick." ; return }
		if {[info exists email]} { putdcc $idx "[searchinfo_dopre]You cannot specify both -dcc and -email." ; return }
		set nick [join [lrange $text [expr $index + 1] [expr $index + 1]]]
	}
	if {[set index [lsearch -exact $text "-flag"]] != -1} { set flags [join [lrange $text [expr $index + 1] [expr $index + 1]]] }
	set csv [expr {([lsearch -exact $text "-csv"] != -1)?"1":"0"}]
	if {[info exists flags]} { set users [userlist $flags] } { set users [userlist] }
	foreach user $users {
		if {[string compare [set data [getuser $user XTRA $type]] ""] == 0 || ([string compare $::searchinfo_setting(ignore) ""] != 0 && [matchattr $user $::searchinfo_setting(ignore)])} { continue }
		if {$csv} {
			if {[string first "$data," $list] == -1} { append list "$data, " }
		} { append list "$user - $data\n" }
	}
	if {![info exists list]} { putdcc $idx "[searchinfo_dopre]No users found." ; return }
	regsub -all -- {[,]$} [string trim $list] {} list
	if {[info exists nick]} {
		if {[catch {open userinfo.txt w} userinfo]} { putdcc $idx "[searchinfo_dopre]Error: Could not open userinfo.txt." ; return }
		puts $userinfo $list ; close $userinfo
		if {[dccsend userinfo.txt $nick] != 0} { putdcc $idx "[searchinfo_dopre]Error: The file could not be sent." } { putdcc $idx "[searchinfo_dopre]Sending file.." }
		if {[catch {file delete -force userinfo.txt}]} { putdcc $idx "[searchinfo_dopre]Error: Could not delete userinfo.txt" }
		return
	}
	if {[info exists email]} {
		if {[catch {exec which mail}]} { putdcc $idx "[searchinfo_dopre]Error: This system does not support 'mail'." ; return }
		if {[catch {open userinfo.tmp w} userinfo]} { putdcc $idx "[searchinfo_dopre]Error: Could not open userinfo.tmp." ; return }
		puts $userinfo $list ; close $userinfo
		if {[catch {exec mail $email -s "SearchInfo" < userinfo.tmp} error]} { putdcc $idx "[searchinfo_dopre]Error: $error" }
		if {[catch {file delete -force userinfo.tmp}]} { putdcc $idx "[searchinfo_dopre]Error: Could not delete userinfo.tmp" }
		return
	}
	foreach line [split $list "\n"] { putdcc $idx "[searchinfo_dopre][join $line]" }
}

proc searchinfo_show {type hand idx text} {
	if {[string compare [set user [join [lrange [split $text] 0 0]]] ""] == 0} { putdcc $idx "[searchinfo_dopre]Usage: .show$type <user>" ; return }
	if {![validuser $user]} { putdcc $idx "[searchinfo_dopre]$user is not a valid user." ; return }
	if {[string compare [set data [getuser $user XTRA $type]] ""] == 0} { putdcc $idx "[searchinfo_dopre]$user does not have [string toupper $type] set." ; return }
	if {[string compare $::searchinfo_setting(ignore) ""] != 0 && [matchattr $user $::searchinfo_setting(ignore)]} { putdcc $idx "[searchinfo_dopre]$user's [string toupper $type] information is hidden." ; return }
	putdcc $idx "[searchinfo_dopre]$user's [string toupper $type] is $data."
}

proc searchinfo_search {type hand idx text} {
	if {[string compare [set string [string tolower [join [lrange [split $text] 0 0]]]] ""] == 0} { putdcc $idx "[searchinfo_dopre]Usage: .search$type <search-string>" ; return }
	set failed 1
	foreach user [userlist] {
		if {[string compare [set data [getuser $user XTRA $type]] ""] == 0 || ![string match [string tolower $text] [string tolower $data]] || ([string compare $::searchinfo_setting(ignore) ""] != 0 && [matchattr $user $::searchinfo_setting(ignore)])} { continue }
		set failed 0
		putdcc $idx "[searchinfo_dopre]$user - $data"
	}
	if {$failed} { putdcc $idx "[searchinfo_dopre]No results found" }
}

putlog "\002SEARCHINFO:\002 SearchInfo.tcl Version 1.0 by Wcc is loaded."